<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\CategoriesResource;
use App\Http\Resources\CertificationsInput;
use App\Http\Resources\CitiesResource;
use App\Http\Resources\CountriesResource;
use App\Http\Resources\MainCategoriesResource;
use App\Http\Resources\SlidersResource;
use App\Http\Resources\SubCategoriesResource;
use App\Http\Resources\WorkersResource;
use App\Models\AlzamaTransfer;
use App\Models\Category;
use App\Models\City;
use App\Models\ContactUs;
use App\Models\Country;
use App\Models\Setting;
use App\Models\Slider;
use App\User;
use Illuminate\Http\Request;
use JWTAuth;

class DataController extends Controller
{
    public function countries()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Country::all();
        return CountriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function cities($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = City::whereCountryId($id)->get();
        return CitiesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function categories()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Category::where('parent_id', null)->get();
        return CategoriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function main_categories()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Category::where('parent_id', null)->get();
        return MainCategoriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }


    public function allSubCategories()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Category::where('parent_id', '!=',null)->get();
        return SubCategoriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }


    public function sub_categories($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Category::where('parent_id', $id)->get();
        return SubCategoriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }


    public function sendTransfer(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'bank_id' => 'required|exists:banks,id',
            'image' => 'required|image'

        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }

            $transfer = new AlzamaTransfer();
            $transfer->user_id = $user->id;
            $transfer->bank_id = $request['bank_id'];
            if ($request->hasFile('image')) $transfer->setImageAttribute();
            $transfer->save();
            return response()->json(['value' => true, 'data' => \Lang::get('data.transfer_done')]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function sliders()
    {
        $rows = Slider::all();
        return SlidersResource::collection($rows)->additional([
            'value' => true,
        ]);
    }

    public function searchCategories(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'search' => 'required'
        ]);
        if ($validator->passes()) {

            $rows = Category::where('parent_id', '!=', 'Null')->where('name->' . \App::getLocale(), 'LIKE', "%{$request->search}%")->get();
            return SubCategoriesResource::collection($rows)->additional([
                'value' => true
            ]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);

        }

    }

    public function certificationsInputs(Request $request)
    {

        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'categories' => 'required'
        ]);
        if ($validator->passes()) {


            $categories = explode(',', $request['categories']);

            $categoriesModel = Category::whereIn('id', $categories)->get();

            return CertificationsInput::collection($categoriesModel)->additional([
                'value' => true
            ]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }


    public function settings()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $setting = Setting::first();
        $data['id'] = $setting->id;
        $data['about'] = $setting->about ? $setting->about[$lang] : '';
        $data['alzama'] = $setting->about ? $setting->alzama[$lang] : '';
        $data['policy'] = $setting->policy ? $setting->policy[$lang] : '';
        $data['advice_tips'] = $setting->advice_tips ? $setting->advice_tips[$lang] : '';
        $data['single_commission'] = $setting->single_commission ? $setting->single_commission : '';
        $data['multiple_commission'] = $setting->multiple_commission ? $setting->multiple_commission : '';
        $data['twitter'] = $setting->twitter ? $setting->twitter : '';
        $data['facebook'] = $setting->facebook ? $setting->facebook : '';
        $data['sanp'] = $setting->sanp ? $setting->sanp : '';
        $data['mobile'] = $setting->mobile ? $setting->mobile : '';
        $data['email'] = $setting->email ? $setting->email : '';
        $data['android_link'] = $setting->android_link ? $setting->android_link : '';
        $data['fee'] = $setting->fee ? $setting->fee : '';
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function contact_us(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'name' => 'required',
            'email' => 'required|email',
            'mobile' => 'required',
            'message' => 'required'
        ]);
        if ($validator->passes()) {
            ContactUs::create($request->all());
            return response()->json(['value' => true, 'data' => \Lang::get('data.sent')]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }
}
