<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\CategoriesResource;
use App\Http\Resources\TransferRequestResource;
use App\Http\Resources\UserResource;
use App\Http\Resources\WorkersResource;
use App\Mail\ResetPassword;
use App\Models\Category;
use App\Models\Certifications;
use App\Models\Order;
use App\Models\Package;
use App\Models\Rate;
use App\Models\Setting;
use App\Models\TransfersRequest;
use App\Models\UserToken;
use App\Models\WorkerCategory;
use App\Models\WorkerImage;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\JWTException;
use JWTAuth;

class UsersController extends Controller
{

    public function register(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'mobile' => ['required', 'unique:users,mobile'],
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'username' => ['required', 'unique:users,username'],
            'email' => 'required|email|unique:users,email',
            'city_id' => 'nullable|exists:cities,id',
            'country_id' => 'nullable|exists:countries,id',
            'type' => 'required|in:worker,client',
            'experience' => 'required_if:type,worker',
            'commercial_number' => 'required_if:type,worker',
            'lat' => 'required',
            'lng' => 'required'

        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['lat'] = $request['lat'];
            $user['lng'] = $request['lng'];
            $user['username'] = $request['username'];
            $user['city_id'] = $request['city_id'];
            $user['country_id'] = $request['country_id'];
            $user['password'] = bcrypt($request['password']);
            $user['type'] = $request['type'];
            $user['experience'] = $request['experience'];
            $user['commercial_number'] = $request['commercial_number'];
            $user['status'] = "not_active";


            $user['code'] = rand(1111, 9999);
            $user->save();

            if ($request['images']) {
                $i = 1;
                foreach ($request['images'] as $image) {

                    $worker_image = new WorkerImage();
                    $worker_image['user_id'] = $user->id;
                    $file_name = $i . time() . '.' . $image->getClientOriginalExtension();
                    $image->move(public_path('images/workers'), $file_name);
                    $worker_image['image'] = $file_name;
                    $worker_image->save();
                    $i++;
                }

            }

            if ($request['type'] == 'worker') {


                $certifications_ids = explode(',', $request['certifications_ids']);
                $certifications = $request['certifications'];


                if (isset($request['certifications']) && count($request['certifications']) > 0) {

                    $certifications_count = count($certifications);


                    for ($i = 0; $i < $certifications_count; $i++) {

                        $cert = new Certifications();
                        $cert->worker_id = $user->id;
                        $cert->category_id = $certifications_ids[$i];
                        $file_name = time() . $i . '.' . $certifications[$i]->getClientOriginalExtension();
                        $certifications[$i]->move(public_path('images/certifications'), $file_name);
                        $cert->certification = $file_name;
                        $cert->save();
                    }

                }


            }


            $msg = " عميلنا العزيز كود التفعيل الخاص بكم هو $user->code ";
            shamel_sms($user->mobile, $msg);

            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                $check->user_id = $user->id;
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->device_language = $lang;
                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'device_language' => $lang
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function activate_user(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required',
            'code' => 'required'
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => "No user with this mobile"]);
            if ($user->code == $request['code']) {
                $user->status = 'active';
                $user->update();
                $data = new UserResource($user);
                return response()->json(['value' => true, 'data' => $data]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code')]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }


    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required',
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'lat' => 'required',
            'lng' => 'required'
            //   'type' => 'required|in:client,worker'

        ]);
        if ($validator->passes()) {

            $auth = ['mobile' => $request->input('mobile'), 'password' => $request->input('password')];
            $user = User::whereMobile($request['mobile'])->first();
            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')], 200);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }

            $user->lat = $request['lat'];
            $user->lng = $request['lng'];
            $user->update();


            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->device_language = $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'device_language' => $lang,
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function logout(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required|exists:tokens,token'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }


            $exist_token = UserToken::where(['token' => $request['device_token']])->first();
            if (!$exist_token) return response()->json(['value' => false, 'msg' => 'device token and auth not match', 'code' => 401]);

            $exist_token->status = 'disconnected';
            $exist_token->update();

            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => \Lang::get('data.logout'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }

    public function update_profile(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'username' => 'unique:users,username,' . $user->id,
            'email' => 'unique:users,email,' . $user->id,
            'mobile' => 'unique:users,mobile,' . $user->id,
            'city_id' => 'exists:cities,id',
            'image' => 'image',
            'experience' => 'nullable'
        ]);
        if ($validator->passes()) {
            $user['username'] = $request['username'] ? $request['username'] : $user->username;
            $user['email'] = $request['email'] ? $request['email'] : $user->email;
            $user['mobile'] = $request['mobile'] ? $request['mobile'] : $user->mobile;
            $user['city_id'] = $request['city_id'] ? $request['city_id'] : $user->city_id;
            $user['experience'] = $request['experience'] ? $request['experience'] : $user->experience;
            if ($request->hasFile('image')) {
                $filename = time() . '.' . $request['image']->getClientOriginalExtension();
                $request['image']->move(public_path('images/users'), $filename);
                $user->image = $filename;
            }
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function forget_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required'
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();

            // mobile code sms
            $msg = " عميلنا العزيز الكود الخاص بكم هو $user->code ";
            shamel_sms($request['mobile'], $msg);
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function check_forget_code(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required|exists:users,code'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            if ($user->code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code'), 'code' => 422]);
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.correct_code'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'password' => 'required|min:6',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',

        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            $user->password = bcrypt($request['password']);
            $user->update();
            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->device_language = $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'device_language' => $lang
                ]);
            }
            $data = new UserResource($user);


            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_old_password(Request $request)
    {

        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'new_password' => 'required|confirmed',
            'new_password_confirmation' => 'required'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }

            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['new_password']);
                $user->update();
                $data = new UserResource($user);
                return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match'), 'code' => 422]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function client_transfers()
    {
        $lang = request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $rows = TransfersRequest::whereUserId($user->id)->paginate(10);
        return TransferRequestResource::collection($rows)->additional(['value' => true, 'code' => 200]);


    }

    public function request_transfer(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'amount' => 'required'
        ]);

        if ($validator->passes()) {

            if ($user->wallet < $request['amount']) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wallet_less_amount')]);
            } else {
                TransfersRequest::create([
                    'user_id' => $user->id,
                    'type' => 'request',
                    'amount' => $request['amount'],
                    'code' => uniqid()
                ]);
                return response()->json(['value' => true, 'data' => \Lang::get('data.sent_to_system')]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }

    }

    public function rate(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'rate' => 'required',
            'comment'
        ]);

        if ($validator->passes()) {

            $order = Order::find($request['order_id']);

            Rate::create([
                'user_id' => $user->id,
                'rated_id' => $order->worker_id,
                'order_id' => $request['order_id'],
                'rate' => $request['rate'],
                'comment' => $request['comment']
            ]);

            return response()->json(['value' => true, 'data' => \Lang::get('data.rated')]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function select_categories(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'category_type' => 'required|in:single,multiple',
            'categories' => 'required'
        ]);
        if ($validator->passes()) {
            $user->category_type = $request['category_type'];
            $user->update();

            if ($request['categories']) {
                foreach (explode(',', $request['categories']) as $category) {
                    WorkerCategory::create([
                        'worker_id' => $user->id,
                        'category_id' => $category
                    ]);
                }
            }
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

        }
    }


    public function worker_categories(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $rows = $user->worker_categories()->paginate(10);

        return CategoriesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function categories_with_check()
    {
        $lng = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lng);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $categories = Category::where('parent_id', '!=', null)->get();
        $data = [];
        foreach ($categories as $category) {
            $arr['id'] = $category->id;
            $arr['name'] = $category->name[$lng];
            $arr['image'] = $category->image;
            if ($user->worker_categories->contains($category)) {
                $status = true;

            } else {
                $status = false;
            }
            $arr['user_category'] = $status;
            $data[] = $arr;
        }
        return response()->json(['value' => true, 'data' => $data]);


    }

    public function update_categories(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'category_type' => 'required|in:single,multiple',
            'categories' => 'required'
        ]);
        if ($validator->passes()) {
            $user->category_type = $request['category_type'];
            $user->update();

            if ($request['categories']) {

                WorkerCategory::whereWorkerId($user->id)->delete();
                foreach (explode(',', $request['categories']) as $category) {
                    WorkerCategory::create([
                        'worker_id' => $user->id,
                        'category_id' => $category
                    ]);
                }
            }
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

        }
    }

    public function worker_profile($id)
    {
        $worker = User::find($id);
        $data = new WorkersResource($worker);
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }
}
